﻿/*
Developed by Kristin Henry (galaxygoo@gmail.com)
This code is available as is, without warranty.

www.underground-flash.com
www.underground-lab.com
August 2008
*/

package{
	import flash.display.MovieClip;
	import flash.display.Sprite;
	import flash.geom.Point;
	import flash.events.MouseEvent;
	import flash.utils.Timer;
	import flash.events.TimerEvent;
	
	
	public class Cell extends MovieClip{
		
		// Properties
		//----------------------------------------
		private var startX:Number;
		private var startY:Number;
		
		public var id:Number;
		public var num:Number;  // atomic number
		public var max:Number;
		public var gMax:Number;
		private var colors:Array = [0x000000, 0xCC99CC, 0x66CC66,
									0x6699CC, 0xCCCCFF, 0x99CC99,
									0x006699, 0x33FF00, 0x009933,
									0x666699];
		public var cellColor;
		
		public var x2;			// width and height of cell
		public var y2;			//
		
		public var s:Number;  		// speed
		public var relPos:Array;	// relative position in table
		public var xy:Array;		// xy coordinates on stage
		
		public var body;
		public var dot;
		public var corner;
		private var offset:Point = new Point();  // offset for faking new registration point
		private var swingDir = 1;
		private var swingTarg = 0;
		
		private var swingTimer:Timer;
		private var fallTimer:Timer;
		private var clickCount;
		
		
					
		// Constructor
		//----------------------------------------
		public function Cell(aNum:Number, w:Number, aMax:Number, gridMax:Number, x1:Number, y1:Number){
			
			num = aNum;
			max = aMax;
			gMax = gridMax;
			
			startX = x1;
			startY = y1;
			
			x2 = w;
			y2 = w;
			
			getID();
			getColor();
			
			relPos = getTablePosition();
			getCorner();
			drawBody();
			s = .1 * id;			
			xy = getXY();
			
			this.x = xy[0];
			this.y = xy[1];
			
			clickCount = 0;
			
			this.addEventListener(MouseEvent.MOUSE_UP, mouseUpListener);
						
		}
		
		// Methods
		//----------------------------------------
		function mouseUpListener(event:MouseEvent){
			
			// move this to frontt of z-order
			this.parent.setChildIndex(this, this.parent.numChildren -1);
						
			this.dot.visible = true;
			
			if(clickCount == 0) {
				
				clickCount = 1;
				swingTimer = new Timer(70, 0);
				swingTimer.addEventListener(TimerEvent.TIMER, swing);
				swingTimer.start();
				
			} else if(clickCount == 1){
				
				swingTimer.stop();
				clickCount = 0;
			}
			
		}
		
		function swing(event:TimerEvent){
			
			var sw = this.body.rotation;
			
			if(sw == this.swingTarg){
				swingTimer.stop();
				this.fallTimerFunction();
			} else {
				this.body.rotation+=5*swingDir;
			}
		}
		
		
		function fallTimerFunction(){
			fallTimer = new Timer(70, 0);
			fallTimer.addEventListener(TimerEvent.TIMER, fall);
			fallTimer.start();
		}
		
		function fall(event:TimerEvent) { 
			this.y+= s * 10;	
		}
		
		function getID(){
			id = Math.floor(10* Math.random()) + 1;
		}
		
		function getColor(){		
			this.cellColor = colors[id -1];//colors[0];
		}
				
				
		function getTablePosition(){
			
			var colMin = 1;
			var colMax = gMax;
			var rowMin = 1;
			var rowMax = gMax;
	
			var col = num % colMax;
			var row = Math.floor(num / rowMax);
				
			return [col, row];
		}
		
		function getXY(){
			var endX = startX + (relPos[0] * x2);
			var endY = startY + (relPos[1] * y2);
			return [endX, endY];
		}
		
		function drawBody(){
			
			var x0 = offset.x;
			var y0 = offset.y;
			var w = x2 - 3;
			var h = y2 - 3;
			
			body = new Sprite();
			body.graphics.beginFill(this.cellColor);
			body.graphics.drawRect(x0,y0,w, h);
			body.graphics.endFill();
			
			body.x = -x0;
			body.y = -y0;
			
			dot = new Sprite();
			dot.graphics.beginFill(0xFF0000);
			dot.graphics.drawCircle(0,0,6);
			dot.graphics.endFill();
			dot.visible = false; 
			body.addChild(dot);
			
			this.addChild(body);
		}
		
		function getCorner(){
			corner = randRange(0,3,0);
			if(corner == 0){
				swingDir = 1;
				swingTarg = 45;
				offset.x = 0;
				offset.y = 0;
			} else if(corner == 1){
				swingDir = -1
				swingTarg = -45;
				offset.x = -x2;
				offset.y = 0;
			} else if(corner == 2){
				swingDir = 1;
				swingTarg = 135;
				offset.x = 0;
				offset.y = -y2;
			} else if(corner == 3){
				swingDir = -1;
				swingTarg = -135;
				offset.x = -x2;
				offset.y = -y2;
			} 
			
		}
		
		//======================================
		// based on function from Joey Lott's ActionScript Cookbook
		function randRange(minNum, maxNum, decPl){
			
			var rangeDiff = (maxNum - minNum) * Math.pow(10, decPl) + 1;
			var randVal = Math.random() * rangeDiff;
			randVal = Math.floor(randVal);
			randVal /= Math.pow(10, decPl);
			randVal += minNum;
	
			return randVal;
		}
		
	}
}